unit LCP100;

interface

uses
  Windows,
  SysUtils, Classes, CPDrv, Graphics;

type
  TLCP100 = class(TComponent)
  private
    { Private-Deklarationen }
    FComPort:TCommPortDriver;
    FPageImages:array[0..63] of TBitmap;
    FBacklightOn: boolean;
    FPageUploading: integer;
    FPageShowing: integer;
    FOnTranferFinished: TNotifyEvent;
    function GetCommPortNo: TPortNumber;
    function GetConnected: boolean;
    function GetPageImage: TBitmap;
    procedure SetBacklightOn(const Value: boolean);
    procedure SetCommPortNo(const Value: TPortNumber);
    procedure SetConnected(const Value: boolean);
    procedure SetPageImage(const Value: TBitmap);
    function GetFirmwareVersion: string;
    procedure SetFirmwareVersion(const Value: string);
    procedure SetPageShowing(const Value: integer);
    procedure SetPageUploading(const Value: integer);
  protected
    { Protected-Deklarationen }
    function Find_LCP100_ComPort:TPortNumber;
  public
    { Public-Deklarationen }
    constructor Create(AnOwner:TComponent); override;
    destructor Destroy; override;
    function Connect:boolean;
    function Disconnect:boolean;

    function ClearDisplay:boolean;
    function ShowTestImage:boolean;
    function UploadImage(Bmp:TBitmap; Page:integer):boolean;
    function ShowPageNo(PageNo:integer):boolean;
  published
    { Published-Deklarationen }
    property CommPortNo:TPortNumber read GetCommPortNo write SetCommPortNo;
    property Connected:boolean read GetConnected write SetConnected stored false;
    property FirmwareVersion:string read GetFirmwareVersion write SetFirmwareVersion stored false;
    property BacklightOn:boolean read FBacklightOn write SetBacklightOn stored false;
    property PageShowing:integer read FPageShowing write SetPageShowing stored false;
    property PageUploading:integer read FPageUploading write SetPageUploading stored false;
    property PageImage:TBitmap read GetPageImage write SetPageImage stored false;
    property OnTranferFinished:TNotifyEvent read FOnTranferFinished write FOnTranferFinished;
  end;

procedure Register;

implementation
uses Registry, LCP100_Cmd;

procedure Register;
begin
  RegisterComponents('Axel', [TLCP100]);
end;

{ TLCP100 }

function TLCP100.ClearDisplay: boolean;
begin
  result := LCP100_ClearDisplay(FComPort);
  if Assigned(FOnTranferFinished) then FOnTranferFinished(Self);
end;

function TLCP100.Connect: boolean;
begin
  result := FComPort.Connect;
  if Result then
  begin
    ClearDisplay;
    BacklightOn := true;
  end;
  if Assigned(FOnTranferFinished) then FOnTranferFinished(Self);
end;

constructor TLCP100.Create(AnOwner: TComponent);
var i:integer;
begin
  inherited;
  FComPort := TCommPortDriver.Create(nil);
  FComPort.BaudRate := br38400;
  FComPort.DataBits := db8BITS;
  FComPort.StopBits := sb1BITS;
  FComPort.Parity := ptEVEN;

  FBacklightOn := true;
  FPageUploading := 0;
  FPageShowing := 0;

  for i := 0 to LCP100_NumberOfPages - 1 do
  begin
    FPageImages[i] := TBitmap.Create;
    FPageImages[i].width := 128;
    FPageImages[i].height := 128;
    FPageImages[i].pixelformat := pf24bit;
  end;
end;

destructor TLCP100.Destroy;
var i:integer;
begin
  for i := 0 to LCP100_NumberOfPages - 1 do
  begin
    FPageImages[i].Free;
  end;

  FComPort.Disconnect;
  FComPort.Free;
  inherited;
end;

function TLCP100.Disconnect: boolean;
begin
  FComPort.Disconnect;
  result := not FComPort.Connected;
end;

function TLCP100.Find_LCP100_ComPort:TPortNumber;
var reg:tregistry;
    sl:tstringlist;
    i:integer;
    portname:string;
begin
  Result := FComPort.Port;
  reg := TRegistry.Create;
  sl := TStringList.Create;
  try
    reg.Access := KEY_READ;
    reg.RootKey := HKEY_LOCAL_MACHINE;
    if reg.OpenKey('Hardware\DeviceMap\SERIALCOMM',false) then
    begin
      reg.GetValueNames(sl);
      sl.text := LowerCase(sl.Text);
      for i := 0 to sl.Count - 1 do
      begin
        if pos('\device\silabser',sl[i]) = 1 then
        begin
          portname := reg.ReadString(sl[i]);
          portname := copy(portname,4,2);
          result := TPortNumber(strtointdef(portname,integer(fcomport.Port)));
          break;
        end;
      end;
    end;
  finally
    reg.Free;
    sl.Free;
  end;
end;

function TLCP100.GetCommPortNo: TPortNumber;
begin
  result := FComPort.Port;
end;

function TLCP100.GetConnected: boolean;
begin
  result := FComPort.Connected;
end;

function TLCP100.GetFirmwareVersion: string;
begin
  result := LCP100_GetFirmwareVersion(FComPort);
  if Assigned(FOnTranferFinished) then FOnTranferFinished(Self);
end;

function TLCP100.GetPageImage: TBitmap;
begin
  result := FPageImages[FPageUploading];
end;

procedure TLCP100.SetBacklightOn(const Value: boolean);
begin
  if value then
  begin
    if LCP100_BacklightOn(FComPort) then FBacklightOn := true;
  end else begin
    if LCP100_BacklightOff(FComPort) then FBacklightOn := false;
  end;
  if Assigned(FOnTranferFinished) then FOnTranferFinished(Self);
end;

procedure TLCP100.SetCommPortNo(const Value: TPortNumber);
begin
  if FComPort.Connected then abort;
  FComPort.Port := Value;
end;

procedure TLCP100.SetConnected(const Value: boolean);
begin
  if value
    then Connect
    else Disconnect;
end;

procedure TLCP100.SetFirmwareVersion(const Value: string);
begin
  //read only
end;

procedure TLCP100.SetPageImage(const Value: TBitmap);
begin
  UploadImage(value,FPageUploading);
end;

procedure TLCP100.SetPageShowing(const Value: integer);
begin
  if (value < 0) or (value > LCP100_NumberOfPages - 1) then exit;

  if LCP100_ShowPage(FComPort, Value)
    then FPageShowing := Value;
  if Assigned(FOnTranferFinished) then FOnTranferFinished(Self);
end;

procedure TLCP100.SetPageUploading(const Value: integer);
begin
  if (value >= 0) and (value <= LCP100_NumberOfPages - 1) then exit;
  FPageUploading := Value;
end;

function TLCP100.ShowPageNo(PageNo: integer): boolean;
begin
  SetPageShowing(PageNo);
  Result := PageNo = FPageShowing;
  if Assigned(FOnTranferFinished) then FOnTranferFinished(Self);
end;

function TLCP100.ShowTestImage: boolean;
begin
  Result := LCP100_TestImage(FComPort);
  if Assigned(FOnTranferFinished) then FOnTranferFinished(Self);
end;

function TLCP100.UploadImage(Bmp: TBitmap; Page: integer): boolean;
begin
  if (Page < 0) or (Page > LCP100_NumberOfPages - 1) then abort;
  FPageImages[Page].Canvas.StretchDraw(FPageImages[Page].Canvas.ClipRect,Bmp);
  Result := LCP100_UploadImage(fcomport,FPageUploading,FPageImages[Page]);
  if Assigned(FOnTranferFinished) then FOnTranferFinished(Self);
end;

end.
